c
c  *** 	Calcuration of Absolute, Relative Response, and Energy Spectra ***
c
c  Parameters:
c   mm: Number of Delta Time for Main Dimension (> nn) 
c   nn: Actual Number of Delta Time
c   dt: Delta Time (dt*nn = duration)
c    h: Dampling Factor
c   sa: Maximum Absolute Response Acceleration (gal)
c   sv: Maximum Relative Response Velocity     (cm/sec)
c  Sva: Maximum Absolute Response Velocity     (cm/sec)
c   sd: Maximum Relative Response Displacement (cm)
c   Ve: Energy Spectra (cm/s) =sqrt(-2*integ(dyy*vel*dt for duration))
c
c  Dimension Data:
c   ddy: Ground Acceleration (gal)
c    dy: Ground Velocity     (cm/s)
c   acc: Absolute Response Acceleration (gal)
c   vel: Relative Response Velocity     (cm/sec)
c  avel: Absolute Response Velocity     (cm/sec)
c   dis: Relative Response Displacement (cm)
c
      program main
      parameter (mm=131100)
      IMPLICIT REAL*8(A-H,O-Z)
      dimension ddy1(mm),ddy2(mm),ddy3(mm),time(mm)
      dimension dy1(mm),dy2(mm),dy3(mm)
      dimension acc(mm),vel(mm),dis(mm)
      CHARACTER INFILE*40,otfile*40,head1*40,head2*40,head3*40,head4*40
c
      write(*,*)
      write(*,*)'Enter File Name for Ground Motion Acceleraration:'
      read(*,1000) infile
 1000 format(a40)
c
      OPEN(11,FILE=INFILE)
        read(11,*)head1,head2,head3,head4
        i=1
   10   read(11,*,end=11)time(i),ddy1(i),ddy2(i),ddy3(i)
        i=i+1
        goto 10
   11 CLOSE(11)
c
      nn=i-1
      dt=time(2)-time(1)
      write(*,*)'nn and DT=',nn,dt
      IF(nn.gt.mm)then
        WRITE(*,*)' nn > mm !!!!!'
        write(*,*)'nn and mm=',nn,mm
        read(*,*)dummy
        STOP
      end if
c
      write(*,*)
      write(*,*)'Enter Damping Factor'
      read(*,*)h
c
c *** Ground Velocity ***
c
      dt2=dt/2.d0
      write(*,*)
      write(*,*)'Ground velocities are seved in "Ground-Vel.csv"'
      write(*,*)'Check the ground velocity waveforms!'
      open(11,file='Ground-Vel.csv')
      dy1(1)=0.d0
      dy2(1)=0.d0
      dy3(1)=0.d0
      write(11,*)'time(s), vel1, vel2, vel3'
      write(11,1500)0.d0,dy1(1),dy2(1),dy3(1)
      do i=2,nn
        dy1(i)=dy1(i-1)+(ddy1(i-1)+ddy1(i))*dt2
        dy2(i)=dy2(i-1)+(ddy2(i-1)+ddy2(i))*dt2
        dy3(i)=dy3(i-1)+(ddy3(i-1)+ddy3(i))*dt2
        write(11,1500)dfloat(i-1)*dt,dy1(i),dy2(i),dy3(i)
      end do
      close(11)
 1500 format(f8.4,3(',',e12.4))
c
c *** Response Spectra ***
c
      write(*,*)
      write(*,*)'Enter Output Response Spectra File Name (*.csv)'
      read(*,1000) otfile
c
      open(11,file=otfile)
      write(11,*)'period(s),Sa1,Sa2,Sa3,pSv1,pSv2,pSv3,Sv1,Sv2,Sv3,Sva1,
     *Sva2,Sva3,Sd1,Sd2,Sd3,Ve1,Ve2,Ve3'
      do 100 j=2,99
        prd=0.01d0*dfloat(j)
	om=6.28318530718d0/prd
      call resp(h,om,dt,nn,ddy1,dy1,acc,vel,dis,mm,sa1,sv1,asv1,sd1,Ve1)
      call resp(h,om,dt,nn,ddy2,dy2,acc,vel,dis,mm,sa2,sv2,asv2,sd2,Ve2)
      call resp(h,om,dt,nn,ddy3,dy3,acc,vel,dis,mm,sa3,sv3,asv3,sd3,Ve3)
      write(11,2000) prd,sa1,sa2,sa3,sa1/om,sa2/om,sa3/om,sv1,sv2,sv3
     *    ,asv1,asv2,asv3,sd1,sd2,sd3,Ve1,Ve2,Ve3
  100 continue
      do 200 j=50,99
        prd=0.02d0*dfloat(j)
	om=6.28318530718d0/prd
      call resp(h,om,dt,nn,ddy1,dy1,acc,vel,dis,mm,sa1,sv1,asv1,sd1,Ve1)
      call resp(h,om,dt,nn,ddy2,dy2,acc,vel,dis,mm,sa2,sv2,asv2,sd2,Ve2)
      call resp(h,om,dt,nn,ddy3,dy3,acc,vel,dis,mm,sa3,sv3,asv3,sd3,Ve3)
      write(11,2000) prd,sa1,sa2,sa3,sa1/om,sa2/om,sa3/om,sv1,sv2,sv3
     *    ,asv1,asv2,asv3,sd1,sd2,sd3,Ve1,Ve2,Ve3
  200 continue
      do 300 j=40,200
        prd=0.05d0*dfloat(j)
	om=6.28318530718d0/prd
      call resp(h,om,dt,nn,ddy1,dy1,acc,vel,dis,mm,sa1,sv1,asv1,sd1,Ve1)
      call resp(h,om,dt,nn,ddy2,dy2,acc,vel,dis,mm,sa2,sv2,asv2,sd2,Ve2)
      call resp(h,om,dt,nn,ddy3,dy3,acc,vel,dis,mm,sa3,sv3,asv3,sd3,Ve3)
      write(11,2000) prd,sa1,sa2,sa3,sa1/om,sa2/om,sa3/om,sv1,sv2,sv3
     *    ,asv1,asv2,asv3,sd1,sd2,sd3,Ve1,Ve2,Ve3
  300 continue
      close(11)
 2000 format(f9.4,18(',',e12.4))

      stop
      end
c
c
c     Subroutine for Response of Single-Degree of System using the Nigam-Jennings method ***
c
      subroutine resp(h,w,dt,nn,ddy,dy,acc,vel,dis,nd,sa,sv,asv,sd,Ve)
c
      IMPLICIT REAL*8(A-H,O-Z)
      dimension ddy(nd),dy(nd),acc(nd),vel(nd),dis(nd)
c
	w2=w*w
	hw=h*w
	wd=w*sqrt(1.d0-h*h)
	wdt=wd*dt
	e=dexp(-hw*dt)
	cwdt=dcos(wdt)
	swdt=dsin(wdt)
c
	a11= e*(cwdt+hW*swdt/wd)
	a12= e*swdt/wd
	a21=-e*w2*swdt/wd
	a22= e*(cwdt-hw*swdt/wd)
	ss=-hw*swdt-wd*cwdt
	cc=-hw*cwdt+wd*swdt
	s1=(e*ss+wd)/w2
	c1=(e*cc+hw)/w2
	s2=(e*dt*ss+hw*s1+wd*c1)/w2
	c2=(e*dt*cc+hw*c1-wd*s1)/w2
	s3=dt*s1-s2
	c3=dt*c1-c2
	b11=-s2/wdt
	b12=-s3/wdt
	b21=(hw*s2-wd*c2)/wdt
	b22=(hw*s3-wd*c3)/wdt
c
	acc(1)=2.d0*h*w*ddy(1)*dt
	vel(1)=-ddy(1)*dt
	dis(1)=0.d0
	dx=vel(1)
	x=0.d0
	sa=0.d0
	sv=0.d0
        asv=0.d0
	sd=0.d0
        Et=0.d0
c
      do 110 m=2,nn
	dxf=dx
	xf=x
	ddym=ddy(m)
	ddyf=ddy(m-1)
c
c     * Relative Response Dis(x) and Vel(dx), and Absolute Acc(ddx) *
	x = a12*dxf+a11*xf+b12*ddym+b11*ddyf
	dx= a22*dxf+a21*xf+b22*ddym+b21*ddyf
	ddx=-2.d0*hw*dx-w2*x
c
c     * Absolute Vel(adx)
	adx= dx+dy(m)
c
	acc(m)=ddx
	vel(m)=dx
	dis(m)=x
c
	sa=dmax1(sa,abs(ddx))
	sv=dmax1(sv,abs(dx))
	asv=dmax1(asv,abs(adx))
	sd=dmax1(sd,abs(x))
c
        Et=Et-(ddyf*dxf+ddym*dx)
c
  110 continue
      EperM=dabs(Et)*dt/2.d0
      Ve=dsqrt(2.d0*EperM)

      return
      end

